<?php

// Exit if accessed directly
if (! defined('ABSPATH')) {
	exit;
}

class WPMeetupOauth2Page extends WPMeetupAdminPage {

	/**
	 * @var WPMeetup
	 */
	var $core;

	public function __construct($core, $set_as_main_admin_page = false) {
		$this->core = $core;
		if ($set_as_main_admin_page) {
			add_menu_page(
				'WP Meetup',
				'WP Meetup',
				'administrator',
				'wp_meetup_oauth2',
				array($this, 'create_page')
			);
		} else {
			add_submenu_page(
				'wp_meetup_settings',
				'OAuth2 Status',
				'OAuth2 Status',
				'administrator',
				'wp_meetup_oauth2',
				array($this, 'create_page')
			);
		}
	}

	public function display_page() {
		$this->check_for_posted_data_to_save();

		$oauth2 = array(
			'key'           => $this->core->options->get_option('oauth2_key'),
			'secret'        => $this->core->options->get_option('oauth2_secret'),
			'access_token'  => $this->core->options->get_option('oauth2_access_token'),
			'token_expires' => $this->core->options->get_option('oauth2_token_expires'),
			'refresh_token' => $this->core->options->get_option('oauth2_refresh_token'),
		);

		// Determine which step we are we on
		if (! empty($oauth2['access_token'])) {
			/*
			 * Step 4 / 4
			 *     - Has access token
			 */
			$this->render_oauth_information();
		} else {

			// Note that Step 3 is taken care of during the `check_for_posted_data_to_save()` method

			if (! empty($oauth2['key']) && ! empty($oauth2['secret'])) {
				/*
				 * Step 2 / 4
				 *     - Key and secret saved
				 *     - No access token
				 *     - No auth code
				 *     - Needs: user to go to secure.meetup.com to sign in and confirm access
				 *     - Data: receives auth code from meetup.com's eventual redirect
				 */
				$this->render_oauth_step_2();
			} else {
				/*
				 * Step 1 / 4
				 *     - Has not started
				 *     - No oauth options saved at all
				 *     - Needs: to enter oauth2 key and secret
				 *     - Data: receives oauth2 key and secret from user-submitted form
				 */
				$this->render_oauth_step_1();
			}

			$this->render_oauth_information();
		}
	}


	/**
	 * A catch-all method which handles posted data from the various steps.
	 *
	 * @since 2.3.0
	 */
	private function check_for_posted_data_to_save() {
		$error_messages = array();
		$success_messages = array();

		if (! empty($_POST['wpm_erase_oauth2_options_nonce']) && wp_verify_nonce($_POST['wpm_erase_oauth2_options_nonce'], 'wpm_erase_oauth2_options')) {
			///
			/// Reset all OAuth2 options
			///
			foreach (
				array(
					'oauth2_key',
					'oauth2_secret',
					'oauth2_access_token',
					'oauth2_token_expires',
					'oauth2_refresh_token'
				) as $key
			) {
				$this->core->options->update_option($key, '');
			}
			$success_messages[] = 'All OAuth2 options have been reset.';
		} else if (! empty($_POST['wpm_oauth2_refresh_access_token_nonce']) && wp_verify_nonce($_POST['wpm_oauth2_refresh_access_token_nonce'], 'wpm_oauth2_refresh_access_token')) {
			///
			/// Manually refresh access_token
			///
			$access_token_refreshed_successfully = $this->core->refresh_access_token();
			if ($access_token_refreshed_successfully) {
				$success_messages[] = 'Successfully refreshed the OAuth2 Access Token!';
			} else {
				$error_messages[] = 'Tried to refresh the OAuth2 Access Token, but was unsuccessful.';
			}
		} else if (! $this->core->options->has_access_token()) {
			if (! empty($_POST['wpm_perform_oauth_authorization_nonce']) && wp_verify_nonce($_POST['wpm_perform_oauth_authorization_nonce'], 'wpm_perform_oauth_authorization')) {
				///
				/// Step 1
				///
				$oauth2_key      = null;
				$oauth2_secret   = null;

				if (! empty($_POST['oauth2_key'])) {
					$oauth2_key = wp_strip_all_tags($_POST['oauth2_key']);
				}
				if (! empty($_POST['oauth2_secret'])) {
					$oauth2_secret = wp_strip_all_tags($_POST['oauth2_secret']);
				}

				if (! empty($oauth2_key) && ! empty($oauth2_secret)) {
					$this->core->options->update_option('oauth2_key', $oauth2_key);
					$this->core->options->update_option('oauth2_secret', $oauth2_secret);

					$success_messages[] = 'Your OAuth2 credentials have been saved. Proceed with step 2.';
				} else {
					$error_messages[] = 'You must submit your OAuth2 key and secret. One or more values was missing.';
				}
			} else if (! empty($_REQUEST['_wpnonce']) && ! empty($_REQUEST['code']) && wp_verify_nonce($_REQUEST['_wpnonce'], 'wpm_receive_authorization_code')) {
				///
				/// Steps 2 and 3
				///
				$authorization_code = wp_strip_all_tags($_REQUEST['code']);
				$access_token_data  = null;
				try {
					$access_token_data = $this->request_access_token_using_authorization_code($authorization_code);
				} catch (Exception $e) {
					$error_messages[] = 'Encountered an error while requesting access token.<br>' . $e->getMessage();
				}

				if (! empty($access_token_data)) {
					$access_token_data = (array) $access_token_data;
					if (! empty($access_token_data['access_token']) && ! empty($access_token_data['expires_in']) && ! empty($access_token_data['refresh_token'])) {
						$this->core->options->update_option('oauth2_access_token', $access_token_data['access_token']);
						$this->core->options->update_option('oauth2_token_expires', $access_token_data['expires_in']);
						$this->core->options->update_option('oauth2_refresh_token', $access_token_data['refresh_token']);
						$this->core->options->update_option('oauth2_token_obtained', time());
						$success_messages[] = sprintf('Access token requested from Meetup and saved. You can now use the WP Meetup plugin! Get started by adding a group in <a href="%s">the Groups page</a>, or go to <a href="%s">the WP Meetup Dashboard page</a>. Enjoy!', admin_url('admin.php?page=wp_meetup_groups'), admin_url('admin.php?page=wp_meetup_settings'));
					} else {
						$error_messages[] = 'Something went wrong while requesting the access token. We expected an array whose keys are <code>access_token</code>, <code>expires_in</code>, and <code>refresh_token</code>. Instead, we got <code>' . json_encode($access_token_data) . '</code>.';
					}
				}
			}
		}
		
		if (! empty($error_messages)) {
			foreach ($error_messages as $msg) {
				echo '<div class="notice notice-error" style="margin: 0 0 15px;"><h2>Error</h2><p>' . $msg . '</p></div>';
			}
		}

		if (! empty($success_messages)) {
			foreach ($success_messages as $msg) {
				echo '<div class="notice notice-success" style="margin: 0 0 15px;"><h2>Success</h2><p>' . $msg . '</p></div>';
			}
		}
	}


	/**
	 * OAuth Authentication Step 1 / 4
	 *     - Has not started
	 *     - No oauth options saved at all
	 *     - Needs: to enter oauth2 key and secret
	 *     - Data: receives oauth2 key and secret from user-submitted form
	 *
	 * @since 2.3.0
	 */
	private function render_oauth_step_1() {
		$this->render_postbox_open('Meetup.com OAuth2 Authentication - Step 1 of 2');
		?>
		<form method="POST" action="">
			<p>To communicate with Meetup.com, you must register a new "OAuth Consumer" in your Meetup.com account. The page to create an OAuth Consumer is located <a href="https://secure.meetup.com/meetup_api/oauth_consumers/" target="_blank">here</a>. You must be signed up for <a href="https://www.meetup.com/pro/" target="_blank">Meetup Pro</a> to register a new OAuth Consumer. There is no alternative API solution for non Meetup Pro (i.e., free-tier) members (that we can figure out or get answers about, in any case).</p>
			<p>Note that when registering your OAuth Consumer, you <strong>must</strong> use <code><?php echo site_url(); ?></code> as the Redirect URI.</p>
			<p>You can read a bit more about this on <a href="https://www.meetup.com/meetup_api/auth/#oauth2" target="_blank">Meetup's OAuth2 documentation page</a>. The plugin authors <strong>do not</strong> provide support for this process. If you have are having any trouble or have any questions, we would encourage you to direct your inquiries to the <a href="https://www.meetup.com/en-us/help?loggedin" target="_blank">Meetup Help Center</a>.</p>
			<table class="form-table">
				<tbody>
					<tr>
						<th scope="row"><label for="oauth2_key">Key</label></th>
						<td><input type="text" class="regular-text" id="oauth2_key" name="oauth2_key" required value=""></td>
					</tr>
					<tr>
						<th scope="row"><label for="oauth2_secret">Secret</label></th>
						<td><input type="text" class="regular-text" id="oauth2_secret" name="oauth2_secret" required value=""></td>
					</tr>
				</tbody>
			</table>
			<?php wp_nonce_field('wpm_perform_oauth_authorization', 'wpm_perform_oauth_authorization_nonce'); ?>
			<button type="submit" class="button button-primary">Submit</button>
		</form>
		<?php
		$this->render_postbox_close();
	}


	/**
	 * OAuth Authentication Step 2 / 4
	 *     - Key and secret saved
	 *     - No access token
	 *     - No auth code
	 *     - Needs: user to go to secure.meetup.com to sign in and confirm access
	 *     - Data: receives auth code from meetup.com's eventual redirect
	 *
	 * @since 2.3.0
	 */
	private function render_oauth_step_2() {
		$oauth2 = array(
			'key'           => $this->core->options->get_option('oauth2_key'),
			'secret'        => $this->core->options->get_option('oauth2_secret'),
		);

		$this->render_postbox_open('Meetup.com OAuth2 Authentication - Step 2 of 2');
		$redirection_nonce = wp_create_nonce('wpm_receive_authorization_code');
		$redirect_uri = admin_url('/admin.php?page=wp_meetup_oauth2&_wpnonce=' . $redirection_nonce);
		$redirect_uri = urlencode($redirect_uri);
		$authorization_url = sprintf('https://secure.meetup.com/oauth2/authorize?client_id=%s&response_type=code&redirect_uri=%s', $oauth2['key'], $redirect_uri);
		?>
		<p>Now that you've created an OAuth consumer, you need to authorize it to access your Meetup account.</p>
		<p>Go to this <a href="<?php echo esc_url($authorization_url); ?>">authorization page</a>, where you will be prompted to Allow the OAuth consumer you created to use your account. Login if needed, then click the Allow button and you will be redirected back to this page.</p>
		<p><a href="<?php echo esc_url($authorization_url); ?>" class="button button-primary">Visit Authorization Page</a></p>
		<?php
		$this->render_postbox_close();
	}


	/**
	 * Request an OAuth2 access token from Meetup.com's API.
	 *
	 * OAuth Authentication Step 3 / 4
	 *     - Key, secret, and auth code saved/available
	 *     - No access token
	 *     - Needs: perform a POST request for an access token
	 *     - Data: receives access token, expiration, and refresh token from POST response
	 *
	 * @since 2.3.0
	 *
	 * @param string $authorization_code
	 *
	 * @return array
	 */
	private function request_access_token_using_authorization_code($authorization_code) {
		$oauth2 = array(
			'key'           => $this->core->options->get_option('oauth2_key'),
			'secret'        => $this->core->options->get_option('oauth2_secret'),
		);
		// We must use the exact same redirect uri (including the "stale" nonce), or the code will be rejected.
		$redirect_uri = admin_url('/admin.php?page=wp_meetup_oauth2&_wpnonce=' . $_REQUEST['_wpnonce']);
		$request_access_token_url = sprintf('https://secure.meetup.com/oauth2/access?client_id=%s&client_secret=%s&grant_type=authorization_code&redirect_uri=%s&code=%s', $oauth2['key'], $oauth2['secret'], urlencode($redirect_uri), urlencode($authorization_code));

		return $this->core->api->remote_post($request_access_token_url, array());
	}


	/**
	 * OAuth Authentication Step 4 / 4
	 *     - Has access token
	 *
	 * This shows the stored oauth2 information.
	 *
	 * @since 2.3.0
	 */
	private function render_oauth_information() {
		$oauth2 = array(
			'key'            => $this->core->options->get_option('oauth2_key'),
			'secret'         => $this->core->options->get_option('oauth2_secret'),
			'access_token'   => $this->core->options->get_option('oauth2_access_token'),
			'token_expires'  => $this->core->options->get_option('oauth2_token_expires'),
			'refresh_token'  => $this->core->options->get_option('oauth2_refresh_token'),
			'token_obtained' => $this->core->options->get_option('oauth2_token_obtained'),
		);

		$this->render_postbox_open('Meetup.com OAuth2 Authentication Information');

		?>
		<?php
		if (! empty($oauth2['access_token'])) {
			$member = $this->core->api->get_member_self();
			if (! empty($member)) {
				printf('<h4>Hello, %s! Your OAuth2 access token is working just great!</h4>', $member->name);
			} else {
				if (empty($member) && ! empty($oauth2['access_token'])) {
					echo '<h4>Warning: it appears as though your stored credentials are not working.</h4>';
				}
			}
		}
		?>
		<p>For debugging and informational purposes:</p>
		<table style="margin: 1em 0">
			<tbody>
				<tr>
					<th align="left" scope="row" valign="top">Key</th>
					<td valign="top"><?php echo esc_html($oauth2['key']); ?></td>
				</tr>
				<tr>
					<th align="left" scope="row" valign="top">Secret</th>
					<td valign="top"><?php echo esc_html($oauth2['secret']); ?></td>
				</tr>
				<tr>
					<th align="left" scope="row" valign="top">Access Token</th>
					<td valign="top"><?php echo esc_html($oauth2['access_token']); ?></td>
				</tr>
				<?php if (! empty($oauth2['token_obtained'])): ?>
					<tr>
						<th align="left" scope="row" valign="top" style="padding-right: 5px;">Access Token Obtained</th>
						<td valign="top">
							Unix Timestamp: <?php echo esc_html($oauth2['token_obtained']); ?>
							<br>
							<?php echo date('r', $oauth2['token_obtained']); ?>
							<br>
							<small><em><?php printf('(%s is the current time)', date('r')); ?></em></small>
						</td>
					</tr>
				<?php endif; ?>
				<?php if (! empty($oauth2['refresh_token'])): ?>
					<tr>
						<th align="left" scope="row" valign="top">Refresh Token</th>
						<td valign="top"><?php echo esc_html($oauth2['refresh_token']); ?></td>
					</tr>
				<?php endif; ?>
			</tbody>
		</table>

		<?php if (! empty($oauth2['refresh_token'])): ?>
			<hr>
			<form method="post">
				<?php wp_nonce_field('wpm_oauth2_refresh_access_token', 'wpm_oauth2_refresh_access_token_nonce'); ?>
				<p><button class="button button-small">Manually Refresh OAuth2 Access Token</button> <abbr title="This is done automatically for you, once per day, so you shouldn't ever need to click this button unless testing something." style="cursor: help;"><span class="dashicons dashicons-editor-help"></span></abbr></p>
			</form>
		<?php endif; ?>
		<hr>
		<form method="post" onsubmit="return confirm('Are you sure you want to erase all your OAuth2 options? You will not be able to undo this process, and must reauthorize with Meetup.')" style="text-align: right">
			<?php wp_nonce_field('wpm_erase_oauth2_options', 'wpm_erase_oauth2_options_nonce'); ?>
			<p><button class="button button-small button-link-delete">Erase All OAuth2 Options</button></p>
		</form>
		<?php
		$this->render_postbox_close();
	}
}
